@@ -9,7 +9,7 @@ use Config::JFDI;
 use Carp qw/croak/;
 use Data::Dumper;
 
-our $VERSION = '0.11';
+our $VERSION = '0.12';
 
 subtype MyConf => as 'HashRef';
 coerce 'MyConf'
@@ -112,11 +112,17 @@ has _position_field => (
                 default  => sub{ 'position' },
                   );
 
+has _rules => (
+                isa      => "HashRef",
+                is       => "rw",
+);
+
 has tree => (
                 isa      => "HashRef",
                 is       => "rw",
 );
 
+
 has root => (
                 isa      => "HashRef",
                 is       => "rw",
@@ -132,6 +138,7 @@ sub _load_backend {
   $self->_log("Loading $backend backend ...");
   $self->load_plugin( $backend );
   $self->_load_tree;
+  $self->_load_rules if ( defined $self->config->{rules});
   $self->_build_tree;
 }
 
@@ -159,6 +166,12 @@ sub _load_tree {
   $self->tree($self->_load);
 }
 
+# Load rules from config
+sub _load_rules {
+  my $self = shift;
+  $self->_rules($self->config->{'rules'});
+}
+
 # Build Tree (children, position, ...)
 # parents and children become HashRef
 sub _build_tree {
@@ -216,6 +229,26 @@ sub  _bckd_update {
     $self->_update($node);
 }
 
+
+sub _check_rules{
+    my ($self, $node, $parent) = @_;
+
+    return 1 if (  ! defined $self->config ||
+                   ! defined $self->config->{rules} ||
+                   $node->{id} == 1
+                );
+
+    # Is node can 'link' to parent ?
+    my $rules = $self->_rules;
+    if ( defined $rules->{$node->{$self->_type_field}}) {
+        if ( defined $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}}) {
+            return $rules->{$node->{$self->_type_field}}->{$parent->{$self->_type_field}};
+        }
+        else { return 0 }
+    }
+    else { return 0 }
+}
+
 sub _update_children_position {
     my $self     = shift;
     my @children = @_;
@@ -243,11 +276,12 @@ sub _remove_child_father{
     my $node = shift;
 
     my $father = $node->{parent};
+    my $key_children = $self->_key_children($node, $father);
     my $id = 0;
-    foreach my $child ( @{$father->{children}}) {
+    foreach my $child ( @{$father->{$key_children}}) {
         if ( $child->{$self->_search_field} eq $node->{$self->_search_field} &&
                  $child->{parent} eq $node->{parent} ){
-            return splice ( @{$father->{children}},$id,1);
+            return splice ( @{$father->{$key_children}},$id,1);
         }
         $id++;
     }
@@ -274,6 +308,41 @@ sub _clone_node {
     return $clone;
 }
 
+sub _key_children {
+    my ($self, $node, $parent) = @_;
+
+    my $key_children;
+    #if parent have a type
+    if ( defined $parent->{$self->_type_field}) {
+
+        # if node have a type
+        if ( defined $node->{$self->_type_field}) {
+
+            # if node and parent have the same type
+            if ( $node->{$self->_type_field} eq $parent->{$self->_type_field} ) {
+                $key_children = 'children';
+            }
+            else {
+                $key_children = 'children_' . $node->{$self->_type_field};
+            }
+        }
+        # else node have the same type as parent
+        else {
+            $node->{$self->_type_field} = $parent->{$self->_type_field};
+            $key_children = 'children';
+        }
+    }
+    # parent haven't type
+    else {
+        if ( defined $node->{$self->_type_field}) {
+            die 'node have a type but not the parent !';
+        }
+        else {
+            $key_children = 'children';
+        }
+    }
+    return $key_children;
+}
 sub search {
   my ( $self, $args, $opts ) = @_;
 
@@ -408,9 +477,10 @@ sub traverse {
     return 0 if ( ! &$funcref( $node, $args ) )
   }
 
-  if(defined($node->{children})) {
+  my $key_children = $self->_key_children($node, $node->{parent});
+  if(defined($node->{$key_children})) {
 
-    foreach my $child ( @{$node->{children}} ) {
+    foreach my $child ( @{$node->{$key_children}} ) {
       return 0 if ( ! &$funcref( $child, $args ) );
       $args->{_count}++;
       $self->traverse( $child, $funcref, $args );
@@ -432,8 +502,9 @@ sub del {
 
       $self->_remove_child_father($node);
 
+      my $key_children = $self->_key_children($node, $node->{parent});
       # if position field exist, recalc all children position
-      $self->_update_children_position( @{$father->{children}} )
+      $self->_update_children_position( @{$father->{$key_children}} )
           if ( defined $node->{$self->_position_field});
 
       # traverse child branches and delete it
@@ -451,6 +522,9 @@ sub add {
   $node->{id}     = $self->_last_node->{id} +1
       if ! defined $node->{id};
 
+  if ( ! $position && defined $node->{$self->_position_field}) {
+      $position = $node->{$self->_position_field};
+  }
 
   if ( ! $parent ){
       if ( $self->root) {
@@ -461,6 +535,10 @@ sub add {
 
   $node->{parent} = $parent;
 
+  # check rules
+  return 0 if ! $self->_check_rules($node, $parent);
+
+
   my $is_exist_in_backend;
   $is_exist_in_backend = 1
       if $self->tree->{$node->{id}};
@@ -468,70 +546,33 @@ sub add {
   # save node in tree
   $self->tree->{$node->{id}} = $node;
 
-
   my $key_children;
-  # node == root
-  if ( ! defined $parent || ! $parent ) {
-      return $self->root;
-  }
-  else {
-      #if parent have a type
-      if ( defined $parent->{$self->_type_field}) {
-
-          # if node have a type
-          if ( defined $node->{$self->_type_field}) {
-              #$self->check_rules($node, $parent);
-
-              # if node and parent have the same type
-              if ( $node->{$self->_type_field} eq $parent->{$self->_type_field} ) {
-                  $key_children = 'children';
-              }
-              else {
-                  $key_children = 'children_' . $node->{$self->_type_field};
-              }
-          }
-          # else node have the same type as parent
-          else {
-              $node->{$self->_type_field} = $parent->{$self->_type_field};
-              $key_children = 'children';
-          }
-      }
-      # parent haven't type
-      else {
-          if ( defined $node->{$self->_type_field}) {
-              die 'node have a type but not the parent !';
-          }
-          else {
-              $key_children = 'children';
-          }
-      }
-
-  }
-
-
-
   my $update_children = 0;
-  if ( $parent ) {
-      if ( $position ) {
-          splice @{$parent->{$key_children}}, $position -1, 0, $self->tree->{$node->{id}};
-          $update_children = 1;
-      }
-      else {
-          push(@{$parent->{$key_children}}, $self->tree->{$node->{id}});
-          if ( defined ${$parent->{$key_children}}[0] && ${$parent->{$key_children}}[0]->{$self->_position_field}) {
+  # node == root
+  if ( defined $parent && $parent ) {
+      $key_children = $self->_key_children($node, $parent);
+
+      if ( $parent ) {
+          if ( $position ) {
+              splice @{$parent->{$key_children}}, $position -1, 0, $self->tree->{$node->{id}};
               $update_children = 1;
           }
+          else {
+              push(@{$parent->{$key_children}}, $self->tree->{$node->{id}});
+              if ( defined ${$parent->{$key_children}}[0] && ${$parent->{$key_children}}[0]->{$self->_position_field}) {
+                  $update_children = 1;
+              }
+          }
       }
   }
-
-
-
   # add node in tree
   $self->_bckd_create($node, 'add node ' . $node->{$self->_search_field})
       if ! $is_exist_in_backend;
 
+  return $self->root if ! $parent;
+
   # recalc position of children
-  $self->_update_children_position(@{$parent->{children}}) if $update_children;
+  $self->_update_children_position(@{$parent->{$key_children}}) if $update_children;
 
   return $node;
 }
@@ -556,7 +597,8 @@ sub update {
 
       my $previous = $node->{$k};
       my $parent   = $node->{parent};
-      my $children = $parent->{children};
+      my $key_children = $self->_key_children($node, $parent);
+      my $children = $parent->{$key_children};
 
 
       # if update 'position'
@@ -565,13 +607,13 @@ sub update {
           my $old = splice(@$children, $previous -1, 1);
           # add child in parent in new positioh
           splice(@$children,$datas->{$k} -1,0, $old);
-          $self->_update_children_position(@{$node->{parent}->{children}})
+          $self->_update_children_position(@{$node->{parent}->{$key_children}})
               if ( defined $node->{$self->_position_field});
       }
       elsif ( $k eq 'parent') {
           my $old = $self->_remove_child_father($node);
           my $new_parent = $datas->{$k};
-          push(@{$new_parent->{children}}, $old);
+          push(@{$new_parent->{$key_children}}, $old);
 
           $node->{parent} = $new_parent;
 
@@ -581,8 +623,8 @@ sub update {
           $self->_bckd_update($node, $msg);
 
           if ( defined $node->{$self->_position_field}) {
-              $self->_update_children_position(@{$parent->{children}});
-              $self->_update_children_position(@{$new_parent->{children}});
+              $self->_update_children_position(@{$parent->{$key_children}});
+              $self->_update_children_position(@{$new_parent->{$key_children}});
           }
       }
       else {
